@@ -10,13 +10,16 @@
 import sys
 import os
 import os.path
+import pickle
+import re
+
 currdir = os.path.dirname(os.path.abspath(__file__))
 import pyutilib.th as unittest
 
 import pyutilib.misc.comparison
 from pyutilib.misc.config import ConfigValue, ConfigBlock, ConfigList
 
-from six import PY3
+from six import PY3, StringIO
 
 try:
     import yaml
@@ -24,6 +27,11 @@ try:
 except ImportError:
     using_yaml = False
 
+# Utility to redirect display() to a string
+def _display(obj, *args):
+    test = StringIO()
+    obj.display(ostream=test, *args)
+    return test.getvalue()
 
 class Test(unittest.TestCase):
 
@@ -295,7 +303,7 @@ flushing:
     max pipes: 2
     response time: 60.0
 """
-        test = self.config.display()
+        test = _display(self.config)
         sys.stdout.write(test)
         self.assertEqual(test, reference)
 
@@ -332,29 +340,29 @@ flushing:
     max pipes: 2
     response time: 60.0
 """
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
-        test = self.config.display()
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
+        test = _display(self.config)
         sys.stdout.write(test)
         self.assertEqual(test, reference)
 
     def test_display_userdata_default(self):
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, "")
 
     def test_display_userdata_list(self):
-        self.config['scenarios'].add()
-        test = self.config.display('userdata')
+        self.config['scenarios'].append()
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios:
   -
 """)
 
     def test_display_userdata_list_nonDefault(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
-        test = self.config.display('userdata')
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios:
   -
@@ -366,7 +374,7 @@ flushing:
     def test_display_userdata_block(self):
         self.config.add("foo", ConfigValue(0, int, None, None))
         self.config.add("bar", ConfigBlock())
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, "")
 
@@ -374,7 +382,7 @@ flushing:
         self.config.add("foo", ConfigValue(0, int, None, None))
         self.config.add("bar", ConfigBlock(implicit=True)) \
                    .add("baz", ConfigBlock())
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, "bar:\n")
 
@@ -390,15 +398,15 @@ flushing:
         self.assertEqual(test, "scenario.merlion")
 
     def test_unusedUserValues_list(self):
-        self.config['scenarios'].add()
+        self.config['scenarios'].append()
         test = '\n'.join(x.name(True) for x in self.config.unused_user_values())
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios
 scenarios[0]""")
 
     def test_unusedUserValues_list_nonDefault(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         test = '\n'.join(x.name(True) for x in self.config.unused_user_values())
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios
@@ -408,8 +416,8 @@ scenarios[1].merlion
 scenarios[1].detection""")
 
     def test_unusedUserValues_list_nonDefault_listAccessed(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         for x in self.config['scenarios']:
             pass
         test = '\n'.join(x.name(True) for x in self.config.unused_user_values())
@@ -420,8 +428,8 @@ scenarios[1].merlion
 scenarios[1].detection""")
 
     def test_unusedUserValues_list_nonDefault_itemAccessed(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         self.config['scenarios'][1]['merlion']
         test = '\n'.join(x.name(True) for x in self.config.unused_user_values())
         sys.stdout.write(test)
@@ -438,7 +446,8 @@ scenarios[1].detection""")
         self.config['scenario'].add('foo', ConfigBlock())
         test = '\n'.join(x.name(True) for x in self.config.unused_user_values())
         sys.stdout.write(test)
-        self.assertEqual(test, "scenario")
+        self.assertEqual(test, """scenario
+scenario.foo""")
 
     def test_UserValues_default(self):
         test = '\n'.join(x.name(True) for x in self.config.user_values())
@@ -452,15 +461,15 @@ scenarios[1].detection""")
         self.assertEqual(test, "scenario.merlion")
 
     def test_UserValues_list(self):
-        self.config['scenarios'].add()
+        self.config['scenarios'].append()
         test = '\n'.join(x.name(True) for x in self.config.user_values())
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios
 scenarios[0]""")
 
     def test_UserValues_list_nonDefault(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         test = '\n'.join(x.name(True) for x in self.config.user_values())
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios
@@ -470,8 +479,8 @@ scenarios[1].merlion
 scenarios[1].detection""")
 
     def test_UserValues_list_nonDefault_listAccessed(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         for x in self.config['scenarios']:
             pass
         test = '\n'.join(x.name(True) for x in self.config.user_values())
@@ -483,8 +492,8 @@ scenarios[1].merlion
 scenarios[1].detection""")
 
     def test_UserValues_list_nonDefault_itemAccessed(self):
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         self.config['scenarios'][1]['merlion']
         test = '\n'.join(x.name(True) for x in self.config.user_values())
         sys.stdout.write(test)
@@ -504,45 +513,46 @@ scenarios[1].detection""")
         self.config['scenario'].add('foo', ConfigBlock())
         test = '\n'.join(x.name(True) for x in self.config.user_values())
         sys.stdout.write(test)
-        self.assertEqual(test, "scenario")
+        self.assertEqual(test, """scenario
+scenario.foo""")
 
     def test_parseDisplayAndValue_default(self):
         if not using_yaml:
             self.skipTest("Cannot execute test because PyYAML is not available")
-        test = self.config.display()
+        test = _display(self.config)
         sys.stdout.write(test)
         self.assertEqual(yaml.load(test), self.config.value())
 
     def test_parseDisplayAndValue_list(self):
         if not using_yaml:
             self.skipTest("Cannot execute test because PyYAML is not available")
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
-        test = self.config.display()
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
+        test = _display(self.config)
         sys.stdout.write(test)
         self.assertEqual(yaml.load(test), self.config.value())
 
     def test_parseDisplay_userdata_default(self):
         if not using_yaml:
             self.skipTest("Cannot execute test because PyYAML is not available")
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(yaml.load(test), None)
 
     def test_parseDisplay_userdata_list(self):
         if not using_yaml:
             self.skipTest("Cannot execute test because PyYAML is not available")
-        self.config['scenarios'].add()
-        test = self.config.display('userdata')
+        self.config['scenarios'].append()
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(yaml.load(test), {'scenarios': [None]})
 
     def test_parseDisplay_userdata_list_nonDefault(self):
         if not using_yaml:
             self.skipTest("Cannot execute test because PyYAML is not available")
-        self.config['scenarios'].add()
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
-        test = self.config.display('userdata')
+        self.config['scenarios'].append()
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
+        test = _display(self.config,'userdata')
         sys.stdout.write(test)
         self.assertEqual(
             yaml.load(test), {'scenarios':
@@ -554,7 +564,7 @@ scenarios[1].detection""")
             self.skipTest("Cannot execute test because PyYAML is not available")
         self.config.add("foo", ConfigValue(0, int, None, None))
         self.config.add("bar", ConfigBlock())
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(yaml.load(test), None)
 
@@ -564,7 +574,7 @@ scenarios[1].detection""")
         self.config.add("foo", ConfigValue(0, int, None, None))
         self.config.add("bar", ConfigBlock(implicit=True)) \
                    .add("baz", ConfigBlock())
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(yaml.load(test), {'bar': None})
 
@@ -579,16 +589,16 @@ scenarios[1].detection""")
         self.assertEqual(val, [])
 
     def test_value_ConfigList_simplePopulated(self):
-        self.config['nodes'].add('1')
-        self.config['nodes'].add(3)
-        self.config['nodes'].add()
+        self.config['nodes'].append('1')
+        self.config['nodes'].append(3)
+        self.config['nodes'].append()
         val = self.config['nodes'].value()
         self.assertIs(type(val), list)
         self.assertEqual(len(val), 3)
         self.assertEqual(val, [1, 3, 0])
 
     def test_value_ConfigList_complexPopulated(self):
-        self.config['scenarios'].add()
+        self.config['scenarios'].append()
         val = self.config['scenarios'].value()
         self.assertIs(type(val), list)
         self.assertEqual(len(val), 1)
@@ -597,7 +607,7 @@ scenarios[1].detection""")
                                 'scenario file': 'Net3.tsg'}])
 
     def test_name(self):
-        self.config['scenarios'].add()
+        self.config['scenarios'].append()
         self.assertEqual(self.config.name(), "")
         self.assertEqual(self.config['scenarios'].name(), "scenarios")
         self.assertEqual(self.config['scenarios'][0].name(), "[0]")
@@ -605,7 +615,7 @@ scenarios[1].detection""")
                          "merlion")
 
     def test_name_fullyQualified(self):
-        self.config['scenarios'].add()
+        self.config['scenarios'].append()
         self.assertEqual(self.config.name(True), "")
         self.assertEqual(self.config['scenarios'].name(True), "scenarios")
         self.assertEqual(self.config['scenarios'][0].name(True), "scenarios[0]")
@@ -623,6 +633,8 @@ scenarios[1].detection""")
             self.config['flushing']['flush nodes']['rate'] = 'a'
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail('expected test to raise ValueError')
         val = self.config['flushing']['flush nodes']['rate']
@@ -646,6 +658,8 @@ scenarios[1].detection""")
             self.config['scenario']['detection'] = 50
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail('expected test to raise ValueError')
         val = self.config['scenario']['detection']
@@ -657,6 +671,8 @@ scenarios[1].detection""")
             self.config['scenario']['detection'] = [5.5, 'a']
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail('expected test to raise ValueError')
         val = self.config['scenario']['detection']
@@ -680,6 +696,8 @@ scenarios[1].detection""")
             self.config['nodes'] = [5, 'a']
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail('expected test to raise ValueError')
         val = self.config['nodes'].value()
@@ -773,6 +791,8 @@ scenarios[1].detection""")
             self.config['scenario'] = _test
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail('expected test to raise ValueError')
         self.assertEqual(self._reference, self.config.value())
@@ -781,6 +801,8 @@ scenarios[1].detection""")
             self.config['scenario'] = []
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail('expected test to raise ValueError')
         self.assertEqual(self._reference, self.config.value())
@@ -804,6 +826,8 @@ scenarios[1].detection""")
             c = ConfigValue('a', domain=int)
         except ValueError:
             pass
+        except:
+            raise
         else:
             self.fail("Expected casting a to int to raise a value error")
 
@@ -850,11 +874,202 @@ scenarios[1].detection""")
                 0])
         os.remove(oFile)
 
+    def test_generate_custom_documentation(self):
+        reference = \
+"""startBlock{}
+  startItem{network}
+  endItem{network}
+  startBlock{network}
+    startItem{epanet file}
+      item{EPANET network inp file}
+    endItem{epanet file}
+  endBlock{network}
+  startItem{scenario}
+    item{Single scenario block}
+  endItem{scenario}
+  startBlock{scenario}
+    startItem{scenario file}
+item{This is the (long) documentation for the 'scenario file'
+parameter.  It contains multiple lines, and some internal
+formatting; like a bulleted list:
+  - item 1
+  - item 2
+}
+    endItem{scenario file}
+    startItem{merlion}
+      item{This is the (long) documentation for the 'merlion' parameter.  It
+      contains multiple lines, but no apparent internal formatting; so the
+      outputter should re-wrap everything.}
+    endItem{merlion}
+    startItem{detection}
+      item{Sensor placement list, epanetID}
+    endItem{detection}
+  endBlock{scenario}
+  startItem{scenarios}
+    item{List of scenario blocks}
+  endItem{scenarios}
+  startBlock{scenarios}
+    startItem{scenario file}
+item{This is the (long) documentation for the 'scenario file'
+parameter.  It contains multiple lines, and some internal
+formatting; like a bulleted list:
+  - item 1
+  - item 2
+}
+    endItem{scenario file}
+    startItem{merlion}
+      item{This is the (long) documentation for the 'merlion' parameter.  It
+      contains multiple lines, but no apparent internal formatting; so the
+      outputter should re-wrap everything.}
+    endItem{merlion}
+    startItem{detection}
+      item{Sensor placement list, epanetID}
+    endItem{detection}
+  endBlock{scenarios}
+  startItem{nodes}
+    item{List of node IDs}
+  endItem{nodes}
+  startItem{impact}
+  endItem{impact}
+  startBlock{impact}
+    startItem{metric}
+      item{Population or network based impact metric}
+    endItem{metric}
+  endBlock{impact}
+  startItem{flushing}
+  endItem{flushing}
+  startBlock{flushing}
+    startItem{flush nodes}
+    endItem{flush nodes}
+    startBlock{flush nodes}
+      startItem{feasible nodes}
+        item{ALL, NZD, NONE, list or filename}
+      endItem{feasible nodes}
+      startItem{infeasible nodes}
+        item{ALL, NZD, NONE, list or filename}
+      endItem{infeasible nodes}
+      startItem{max nodes}
+        item{Maximum number of nodes to flush}
+      endItem{max nodes}
+      startItem{rate}
+        item{Flushing rate [gallons/min]}
+      endItem{rate}
+      startItem{response time}
+        item{Time [min] between detection and flushing}
+      endItem{response time}
+      startItem{duration}
+        item{Time [min] for flushing}
+      endItem{duration}
+    endBlock{flush nodes}
+    startItem{close valves}
+    endItem{close valves}
+    startBlock{close valves}
+      startItem{feasible pipes}
+        item{ALL, DIAM min max [inch], NONE, list or filename}
+      endItem{feasible pipes}
+      startItem{infeasible pipes}
+        item{ALL, DIAM min max [inch], NONE, list or filename}
+      endItem{infeasible pipes}
+      startItem{max pipes}
+        item{Maximum number of pipes to close}
+      endItem{max pipes}
+      startItem{response time}
+        item{Time [min] between detection and closing valves}
+      endItem{response time}
+    endBlock{close valves}
+  endBlock{flushing}
+endBlock{}
+"""
+        test = self.config.generate_documentation(
+            block_start= "startBlock{%s}\n",
+            block_end=   "endBlock{%s}\n",
+            item_start=  "startItem{%s}\n",
+            item_body=   "item{%s}\n",
+            item_end=    "endItem{%s}\n",
+        )
+        print(test)
+        self.assertEqual(test, reference)
+
+        test = self.config.generate_documentation(
+            block_start= "startBlock\n",
+            block_end=   "endBlock\n",
+            item_start=  "startItem\n",
+            item_body=   "item\n",
+            item_end=    "endItem\n",
+        )
+
+        # re.sub(flags=) was introduced in 2.7
+        if sys.version_info[:2] < (2,7):
+            stripped_reference = re.sub('\{[^\}]*\}','',reference)
+        else:
+            stripped_reference = re.sub('\{[^\}]*\}','',reference,flags=re.M)
+        print(test)
+        self.assertEqual(test, stripped_reference)
+
     def test_block_get(self):
         self.assertTrue('scenario' in self.config)
-        self.assertNotEquals(self.config.get('scenario', 'bogus'), 'bogus')
+        self.assertNotEquals(
+            self.config.get('scenario', 'bogus').value(), 'bogus')
         self.assertFalse('fubar' in self.config)
-        self.assertEquals(self.config.get('fubar', 'bogus'), 'bogus')
+        self.assertEquals(
+            self.config.get('fubar', 'bogus').value(), 'bogus')
+
+        cfg = ConfigBlock()
+        cfg.declare('foo', ConfigValue(1, int))
+        self.assertEqual( cfg.get('foo', 5).value(), 1 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertIs( cfg.get('bar'), None )
+        self.assertEqual( cfg.get('bar',None).value(), None )
+        self.assertEqual( len(cfg), 1 )
+
+        cfg = ConfigBlock(implicit=True)
+        cfg.declare('foo', ConfigValue(1, int))
+        self.assertEqual( cfg.get('foo', 5).value(), 1 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertEqual( cfg.get('bar', 5).value(), 5 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertIs( cfg.get('baz'), None )
+        self.assertIs( cfg.get('baz', None).value(), None )
+        self.assertEqual( len(cfg), 1 )
+
+        cfg = ConfigBlock( implicit=True,
+                           implicit_domain=ConfigList(domain=str) )
+        cfg.declare('foo', ConfigValue(1, int))
+        self.assertEqual( cfg.get('foo', 5).value(), 1 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertEqual( cfg.get('bar', [5]).value(), ['5'] )
+        self.assertEqual( len(cfg), 1 )
+        self.assertIs( cfg.get('baz'), None )
+        self.assertEqual( cfg.get('baz', None).value(), [] )
+        self.assertEqual( len(cfg), 1 )
+
+    def test_setdefault(self):
+        cfg = ConfigBlock()
+        cfg.declare('foo', ConfigValue(1, int))
+        self.assertEqual( cfg.setdefault('foo', 5).value(), 1 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertRaisesRegexp(ValueError, '.*disallows implicit entries',
+                                cfg.setdefault, 'bar', 0)
+        self.assertEqual( len(cfg), 1 )
+
+        cfg = ConfigBlock(implicit=True)
+        cfg.declare('foo', ConfigValue(1, int))
+        self.assertEqual( cfg.setdefault('foo', 5).value(), 1 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertEqual( cfg.setdefault('bar', 5).value(), 5 )
+        self.assertEqual( len(cfg), 2 )
+        self.assertEqual( cfg.setdefault('baz').value(), None )
+        self.assertEqual( len(cfg), 3 )
+
+        cfg = ConfigBlock( implicit=True,
+                           implicit_domain=ConfigList(domain=str) )
+        cfg.declare('foo', ConfigValue(1, int))
+        self.assertEqual( cfg.setdefault('foo', 5).value(), 1 )
+        self.assertEqual( len(cfg), 1 )
+        self.assertEqual( cfg.setdefault('bar', [5]).value(), ['5'] )
+        self.assertEqual( len(cfg), 2 )
+        self.assertEqual( cfg.setdefault('baz').value(), [] )
+        self.assertEqual( len(cfg), 3 )
 
     def test_block_keys(self):
         ref = ['scenario file', 'merlion', 'detection']
@@ -931,11 +1146,11 @@ scenarios[1].detection""")
 
     def test_list_manipulation(self):
         self.assertEqual(len(self.config['scenarios']), 0)
-        self.config['scenarios'].add()
+        self.config['scenarios'].append()
         self.assertEqual(len(self.config['scenarios']), 1)
-        self.config['scenarios'].add({'merlion': True, 'detection': []})
+        self.config['scenarios'].append({'merlion': True, 'detection': []})
         self.assertEqual(len(self.config['scenarios']), 2)
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios:
   -
@@ -945,7 +1160,7 @@ scenarios[1].detection""")
 """)
         self.config['scenarios'][0] = {'merlion': True, 'detection': []}
         self.assertEqual(len(self.config['scenarios']), 2)
-        test = self.config.display('userdata')
+        test = _display(self.config, 'userdata')
         sys.stdout.write(test)
         self.assertEqual(test, """scenarios:
   -
@@ -955,7 +1170,7 @@ scenarios[1].detection""")
     merlion: true
     detection: []
 """)
-        test = self.config['scenarios'].display()
+        test = _display(self.config['scenarios'])
         sys.stdout.write(test)
         self.assertEqual(test, """-
   scenario file: Net3.tsg
@@ -967,6 +1182,53 @@ scenarios[1].detection""")
   detection: []
 """)
 
+    def test_list_get(self):
+        X = ConfigBlock(implicit=True)
+        X.config = ConfigList()
+        self.assertEqual(_display(X, 'userdata'), "")
+        self.assertIs(X.config.get(0), None )
+        self.assertIs(X.config.get(0,None).value(), None )
+        self.assertRaisesRegexp(
+            IndexError, '.*out of range', X.config.__getitem__, 0 )
+        # get() shouldn't change the userdata flag...
+        self.assertEqual(_display(X, 'userdata'), "")
+
+        X = ConfigBlock(implicit=True)
+        X.config = ConfigList([42], int)
+        self.assertEqual(_display(X, 'userdata'), "")
+        val = X.config.get(0)
+        self.assertEqual(val.value(), 42)
+        self.assertIs(type(val), ConfigValue)
+        # get() shouldn't change the userdata flag...
+        self.assertEqual(_display(X, 'userdata'), "")
+        val = X.config[0]
+        self.assertIs(type(val), int)
+        self.assertEqual(val, 42)
+        # get() shouldn't change the userdata flag...
+        self.assertEqual(_display(X, 'userdata'), "")
+
+        self.assertIs(X.config.get(1), None )
+        self.assertRaisesRegexp(
+            IndexError, '.*out of range', X.config.__getitem__, 1)
+
+        # this should ONLY change the userSet flag on the item (and not
+        # the list)
+        X.config.get(0).set_value(20)
+        self.assertEqual(_display(X, 'userdata'), "  - 20\n")
+
+        # this should ONLY change the userSet flag on the item (and not
+        # the list)
+        X = ConfigBlock(implicit=True)
+        X.config = ConfigList([42], int)
+        X.config[0] = 20
+        self.assertEqual(_display(X, 'userdata'), "  - 20\n")
+
+        # This should change both...
+        X = ConfigBlock(implicit=True)
+        X.config = ConfigList([42], int)
+        X.config.append(20)
+        self.assertEqual(_display(X, 'userdata'), "config:\n  - 20\n")
+
     def test_implicit_entries(self):
         config = ConfigBlock()
         try:
@@ -982,7 +1244,6 @@ scenarios[1].detection""")
         config['implicit_1'] = 5
         config.declare('formal', ConfigValue(42, int))
         config['implicit_2'] = 5
-        print(config.display())
         self.assertEqual(3, len(config))
         self.assertEqual(['implicit_1', 'formal', 'implicit_2'],
                          list(config.iterkeys()))
@@ -1170,5 +1431,56 @@ Node information:
                 "Expected implicit assignment to explicit block to raise ValueError")
 
 
+    def test_nonString_keys(self):
+        config = ConfigBlock(implicit=True)
+        config.declare(5, ConfigValue(50, int))
+        self.assertIn(5, config)
+        self.assertIn('5', config)
+        self.assertEqual(config[5], 50)
+        self.assertEqual(config['5'], 50)
+        self.assertEqual(config.get(5).value(), 50)
+        self.assertEqual(config.get('5').value(), 50)
+
+        config[5] = 500
+        self.assertIn(5, config)
+        self.assertIn('5', config)
+        self.assertEqual(config[5], 500)
+        self.assertEqual(config['5'], 500)
+        self.assertEqual(config.get(5).value(), 500)
+        self.assertEqual(config.get('5').value(), 500)
+
+        config[1] = 10
+        self.assertIn(1, config)
+        self.assertIn('1', config)
+        self.assertEqual(config[1], 10)
+        self.assertEqual(config['1'], 10)
+        self.assertEqual(config.get(1).value(), 10)
+        self.assertEqual(config.get('1').value(), 10)
+
+        self.assertEqual(_display(config), "5: 500\n1: 10\n")
+
+        config.set_value({5:5000})
+        self.assertIn(1, config)
+        self.assertIn('1', config)
+        self.assertEqual(config[1], 10)
+        self.assertEqual(config['1'], 10)
+        self.assertEqual(config.get(1).value(), 10)
+        self.assertEqual(config.get('1').value(), 10)
+        self.assertIn(5, config)
+        self.assertIn('5', config)
+        self.assertEqual(config[5], 5000)
+        self.assertEqual(config['5'], 5000)
+        self.assertEqual(config.get(5).value(), 5000)
+        self.assertEqual(config.get('5').value(), 5000)
+
+    def test_pickle(self):
+        # A simple test: pickle the default config and restore it.
+        # calling other tests should still pass.
+        pickle_str = pickle.dumps(self.config)
+        self.config = None
+        self.config = pickle.loads(pickle_str)
+        self.test_display_list()
+
+
 if __name__ == "__main__":
     unittest.main()
