@@ -17,7 +17,6 @@ import os
 import sys
 import tempfile
 import subprocess
-import copy
 from six import itervalues
 from threading import Thread
 
@@ -196,7 +195,18 @@ def _stream_reader(args):
             except ValueError:
                 pass
 
-    encoding = sys.__stdout__.encoding
+    raw_stderr = sys.__stderr__
+    if raw_stderr is None:
+        raw_stderr = sys.stderr
+    try:
+        encoding = stream.encoding
+    except:
+        encoding = None
+    if encoding is None:
+        try:
+            encoding = raw_stderr.encoding
+        except:
+            pass
     if encoding is None:
         encoding = 'utf-8'
 
@@ -234,14 +244,14 @@ def _stream_reader(args):
     if data:
         writeOK &= write(data.decode(encoding))
     flush()
-    if not writeOK:
-        sys.__stderr__.write("""
+    if not writeOK and raw_stderr is not None:
+        raw_stderr.write("""
 ERROR: pyutilib.subprocess: output stream closed before all subprocess output
        was written to it.  The following was left in the subprocess buffer:
             '%s'
 """ % (buf,))
         if data:
-            sys.__stderr__.write(
+            raw_stderr.write(
                 """The following undecoded unicode output was also present:
             '%s'
 """ % (data,))
@@ -289,7 +299,21 @@ def _merged_reader(*args):
                 except ValueError:
                     pass
 
-    encoding = sys.__stdout__.encoding
+    raw_stderr = sys.__stderr__
+    if raw_stderr is None:
+        # There are cases, e.g., in Anaconda, where there is no stdout
+        # for the original process because, for example, it was started
+        # in a windowing environment.
+        raw_stderr = sys.stderr
+    try:
+        encoding = stream.encoding
+    except:
+        encoding = None
+    if encoding is None:
+        try:
+            encoding = raw_stderr.encoding
+        except:
+            pass
     if encoding is None:
         encoding = 'utf-8'
 
@@ -351,14 +375,14 @@ def _merged_reader(*args):
         if s.data:
             writeOK &= s.write(s.data.decode(encoding))
         s.flush()
-    if not writeOK:
-        sys.__stderr__.write("""
+    if not writeOK and raw_stderr is not None:
+        raw_stderr.write("""
 ERROR: pyutilib.subprocess: output stream closed before all subprocess output
        was written to it.  The following was left in the subprocess buffer:
             '%s'
 """ % (buf,))
         if data:
-            sys.__stderr__.write(
+            raw_stderr.write(
                 """The following undecoded unicode output was also present:
             '%s'
 """ % (data,))
@@ -492,7 +516,7 @@ def run_command(cmd,
     # Setup the default environment
     #
     if env is None:
-        env = copy.copy(os.environ)
+        env = os.environ.copy()
     #
     # Setup signal handler
     #
@@ -520,6 +544,7 @@ def run_command(cmd,
         except:
             simpleCase = False
 
+        out_th = []
         GlobalData.signal_handler_busy = False
         if simpleCase:
             #
@@ -542,7 +567,6 @@ def run_command(cmd,
             # as doing a normal 'print'
             #
             out_fd = []
-            out_th = []
             for fid in (0, 1):
                 if fid == 0:
                     s, raw = stdout_arg, sys.stdout
@@ -608,34 +632,12 @@ def run_command(cmd,
                 th = Thread(target=reader, args=[x[0] for x in out_th])
                 th.daemon = True
                 th.start()
-            #for th in out_th:
-            #    th[0].daemon = True
-            #    th[0].start()
             #
             # Wait for process to finish
             #
             rc = process.wait(timelimit)
             GlobalData.current_process = None
             out_fd = None
-            #
-            # 'Closing' the PIPE to send EOF to the reader.
-            #
-            if out_th:
-                for p in out_th:
-                    os.close(p[2])
-                th.join()
-                for p in out_th:
-                    os.close(p[1])
-                del th
-            #for th in reversed(out_th):
-            #    os.close(th[2])
-            #    #
-            #    # Wait for readers to finish up with the data in the pipe.
-            #    #
-            #    th[0].join()
-            #    os.close(th[1])
-            #    thread = th[0]
-            #    del thread
 
     except _WindowsError:
         err = sys.exc_info()[1]
@@ -647,14 +649,37 @@ def run_command(cmd,
         # Ignore IOErrors, which are caused by interupts
         #
         pass
+
+    #
+    # Flush stdout/stderr. Some platforms (notably Matlab, which
+    # replaces stdout with a MexPrinter) have stdout/stderr that do not
+    # implement flush()  See https://github.com/Pyomo/pyomo/issues/156
+    #
+    try:
+        sys.stdout.flush()
+    except AttributeError:
+        pass
+    try:
+        sys.stderr.flush()
+    except AttributeError:
+        pass
+
+    if out_th:
+        #
+        # 'Closing' the PIPE to send EOF to the reader.
+        #
+        for p in out_th:
+            os.close(p[2])
+        th.join()
+        for p in out_th:
+            os.close(p[1])
+        del th
     if outfile is not None:
         stdout_arg.close()
     elif tmpfile is not None and not ignore_output:
         tmpfile.seek(0)
         output = "".join(tmpfile.readlines())
         tmpfile.close()
-    sys.stdout.flush()
-    sys.stderr.flush()
     #
     # Move back from the specified working directory
     #
